import { h } from "@stencil/core";
import datepicker from "js-datepicker";
import { overrideSettings } from "../../utils/utils";
export class FdiDateTimePicker {
    constructor() {
        this.__settings = undefined;
        this.__datetime = undefined;
        this.__dateonly = false;
        this.picker = undefined;
        this.date = undefined;
        this.time = '1:00';
        this.station = 'PM';
        this.timezone = 'None';
        this.settings = {
            meta: {}
        };
    }
    componentDidLoad() {
        const datetime = new Date();
        const [date, time, station, timezone] = this.__datetime ? this.__datetime.split(' ') : [undefined, undefined, undefined, undefined];
        if (time) {
            this.time = time;
        }
        if (station) {
            this.station = station;
        }
        if (timezone) {
            this.timezone = timezone;
        }
        const today = new Date(datetime.getFullYear(), datetime.getMonth(), datetime.getDate());
        this.date = date ? date : today.toLocaleDateString();
        this.picker = datepicker(this.element.shadowRoot.getElementById('datepicker'), {
            minDate: today,
            dateSelected: this.date ? new Date(this.date) : today,
            formatter: (input, date) => {
                input.value = date.toLocaleDateString();
            },
            onSelect: (instance, date) => {
                this.date = date.toLocaleDateString();
                console.log({ value: this.getValue(), instance });
                this.dateTimeChanged.emit({
                    newValue: this.getValue(),
                    meta: this.settings.meta,
                });
            }
        });
        this.dateTimeChanged.emit({
            newValue: this.getValue(),
            meta: this.settings.meta,
        });
    }
    getValue() {
        return this.__dateonly ? this.date.trim() : `${this.date} ${this.time} ${this.station} ${this.timezone}`.trim();
    }
    handleTimezoneChange(e) {
        this.timezone = e.target.value;
        this.dateTimeChanged.emit({
            newValue: this.getValue(),
            meta: this.settings.meta,
        });
    }
    handleStationChange(e) {
        this.station = e.target.value;
        this.dateTimeChanged.emit({
            newValue: this.getValue(),
            meta: this.settings.meta,
        });
    }
    handleTimeChange(e) {
        this.time = e.target.value;
        this.dateTimeChanged.emit({
            newValue: this.getValue(),
            meta: this.settings.meta,
        });
    }
    componentWillLoad() {
        let settings = this.__settings;
        if (typeof settings === 'string') {
            settings = JSON.parse(this.__settings);
        }
        this.settings = overrideSettings(this.settings, settings);
    }
    render() {
        const availableTimes = [];
        const hours = 12;
        const minutes = [
            '00',
            15,
            30,
            45
        ];
        for (let i = 1; i <= hours; i++) {
            minutes.forEach(minute => {
                availableTimes.push(`${i}:${minute}`);
            });
        }
        return (h("div", { key: '15bc009c8c48d1e040a6da472f2e994eea788c47' }, h("input", { key: '4e68b21081d970e917473b17bd3abe733a97e93e', id: 'datepicker', placeholder: 'Select Date' }), !this.__dateonly &&
            h("select", { key: '5fcc2c88bf427e2abab751955d93eeb974934470', name: "time", id: "time", onInput: e => { this.handleTimeChange(e); } }, availableTimes.map(time => h("option", { value: time, selected: this.time === time }, time))), !this.__dateonly &&
            h("select", { key: 'c72cff565297b706a8e9614562390aaba91db764', name: "station", id: "station", onInput: e => { this.handleStationChange(e); } }, h("option", { key: 'b7c35cc2c29066a78a1e3f90ce2dc4004dfc29ac', value: 'PM', selected: this.station === 'PM' }, "PM"), h("option", { key: 'fd6a88a114daeb022137b9e6235593a02a10c7be', value: 'AM', selected: this.station === 'AM' }, "AM")), !this.__dateonly &&
            h("select", { key: '27898f39e38bf19a9bc7649d32622aacc144c25a', name: "timezone", id: "timezone", onInput: e => { this.handleTimezoneChange(e); } }, h("option", { key: '1f501e315f9538a6d238cedb1a1ef35f48d84be1', value: 'None', selected: this.timezone === 'None' }, "No Timezone"), h("option", { key: '3423fa2b50deb9d7e54684ac8c47c173c69c12e3', value: 'America/Los_Angeles', selected: this.timezone === 'America/Los_Angeles' }, "Pacific"), h("option", { key: '772353fb6ed9f4752b552e7b5abac9fb2b61e8d9', value: 'America/New_York', selected: this.timezone === 'America/New_York' }, "Eastern"), h("option", { key: '6c42d13b1373c28f5327e472cead966f6b3ad9fd', value: 'America/Denver', selected: this.timezone === 'America/Denver' }, "Mountain"), h("option", { key: '54666e4f1a9eb9e7ae4ca787107aacb8c4af2d14', value: 'America/Chicago', selected: this.timezone === 'America/Chicago' }, "Central"), h("option", { key: 'd71b70f1924b1d9c5034c42801240135b747c03f', value: 'America/Anchorage', selected: this.timezone === 'America/Anchorage' }, "Alaskan"), h("option", { key: 'ecd5f1d73f27b591693a97d1099374b5321bd459', value: 'America/Adak', selected: this.timezone === 'America/Adak' }, "Hawaiian"), h("option", { key: '66ea4949c8af6b86b73e88806a96504915213a42', value: 'America/Halifax', selected: this.timezone === 'America/Halifax' }, "Atlantic"), h("option", { key: '1af6f201e30cbeee2b314029e412dfd59f20075a', value: 'America/St_Johns', selected: this.timezone === 'America/St_Johns' }, "New Foundland"))));
    }
    static get is() { return "fdi-date-time-picker"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["fdi-date-time-picker.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["fdi-date-time-picker.css"]
        };
    }
    static get properties() {
        return {
            "__settings": {
                "type": "any",
                "mutable": false,
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__settings",
                "reflect": false
            },
            "__datetime": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__datetime",
                "reflect": false,
                "defaultValue": "undefined"
            },
            "__dateonly": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__dateonly",
                "reflect": false,
                "defaultValue": "false"
            }
        };
    }
    static get states() {
        return {
            "picker": {},
            "date": {},
            "time": {},
            "station": {},
            "timezone": {},
            "settings": {}
        };
    }
    static get events() {
        return [{
                "method": "dateTimeChanged",
                "name": "fdi_wc_datetimepicker_change",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }];
    }
    static get elementRef() { return "element"; }
}
//# sourceMappingURL=fdi-date-time-picker.js.map
