import { h } from "@stencil/core";
import { overrideSettings } from "../../utils/utils";
export class FdiImageUploader {
    constructor() {
        this.__settings = undefined;
        this.__value = undefined;
        this.src = undefined;
        this.settings = {
            meta: {},
        };
    }
    emitData() {
        this.imageUploader.emit({
            src: this.src,
            meta: this.settings.meta,
        });
    }
    componentWillLoad() {
        let settings = this.__settings;
        if (typeof settings === 'string') {
            settings = JSON.parse(settings);
        }
        if (this.__value) {
            this.src = this.__value;
        }
        this.settings = overrideSettings(this.settings, settings);
    }
    selectImage() {
        // @ts-ignore
        let wordpress = wp || null;
        console.log({ wordpress });
        if (wordpress) {
            let image_frame = wordpress.media({
                title: 'Select Image',
                multiple: false,
                library: {
                    type: 'image'
                }
            });
            image_frame.on('open', function () {
                var lastSelection = image_frame.state().get('selection');
                const attach = wordpress.media.attachment(lastSelection._single.id);
                if (attach) {
                    attach.fetch();
                }
                lastSelection.add(attach ? [attach] : []);
            });
            image_frame.on('select', function () {
                let selection = image_frame.state().get('selection');
                var metadata = image_frame.state().get('selection').first().toJSON();
                console.log({ selection, metadata });
                selection.each(function (attachment) {
                    let attrs = attachment["attributes"];
                    if (!attrs.alt) {
                        alert(`Alt text is required for ${attrs.filename} to select it.`);
                        image_frame.open();
                    }
                    else if (metadata.url) {
                        this.src = {
                            media_library: {
                                id: metadata.id,
                                src: metadata.url
                            }
                        };
                        this.emitData();
                    }
                }.bind(this));
            }.bind(this));
            image_frame.open();
        }
    }
    render() {
        var _a, _b;
        return (h("div", { key: '6a9d4819064cfa020972c5ebf4b13c135e532010' }, h("div", { key: '1abfff9dc6fa66edb5273806aa7cabbe8abc6ca4' }, h("button", { key: '6bf3c57dc5284f217e689ddd619290db66df5c39', type: "button", onClick: () => this.selectImage() }, "Select Image")), h("div", { key: 'e4f834402aeadb64d3c30b3af9c914977f6117f6', id: "image_uploader_preview" }, ((_b = (_a = this === null || this === void 0 ? void 0 : this.src) === null || _a === void 0 ? void 0 : _a.media_library) === null || _b === void 0 ? void 0 : _b.src) && h("img", { key: 'f4c88a4b35528f482ff29e12dddb06b08c9abe81', style: { maxHeight: '100%', maxWidth: '100%' }, src: this.src.media_library.src }))));
    }
    static get is() { return "fdi-image-uploader"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["fdi-image-uploader.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["fdi-image-uploader.css"]
        };
    }
    static get properties() {
        return {
            "__settings": {
                "type": "any",
                "mutable": false,
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__settings",
                "reflect": false
            },
            "__value": {
                "type": "any",
                "mutable": false,
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__value",
                "reflect": false
            }
        };
    }
    static get states() {
        return {
            "src": {},
            "settings": {}
        };
    }
    static get events() {
        return [{
                "method": "imageUploader",
                "name": "fdi_wc_image_uploader_load",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }];
    }
}
//# sourceMappingURL=fdi-image-uploader.js.map
