import { h, forceUpdate } from "@stencil/core";
import { overrideSettings } from "../../utils/utils";
import { v4 as uuidv4 } from "uuid";
export class FdiMultiInput {
    constructor() {
        this.__inputs = '[]';
        this.__settings = undefined;
        this.inputs = undefined;
        this.settings = {
            inputSettings: {},
            containerStyle: {},
            containerClass: 'fdi wc multi-input-container',
            inputContainerStyle: {},
            inputContainerClass: 'input-container',
            buttonContainerStyle: {},
            buttonContainerClass: 'button-container',
            deleteButtonStyle: {},
            deleteButtonClass: 'delete-button',
            addButtonStyle: {},
            addButtonClass: 'add-button',
            meta: {},
        };
    }
    fdi_wc_input_change__handler(ev) {
        const { newValue, meta: { key } } = ev.detail;
        const existingInput = this.inputs.get(key);
        if (existingInput) {
            this.inputs.set(key, Object.assign(Object.assign({}, existingInput), { __value: newValue }));
        }
        this.emitValue();
    }
    emitValue() {
        const values = [];
        this.inputs.forEach(({ __value }) => {
            values.push(__value);
        });
        this.multiInputChanged.emit({
            newValue: values,
            meta: this.settings.meta,
        });
    }
    removeInput(key) {
        this.inputs.delete(key);
        this.emitValue();
        forceUpdate(this);
    }
    addInput(value = '') {
        const key = uuidv4();
        this.inputs.set(key, {
            style: { flexGrow: 1, display: 'flex' },
            __settings: Object.assign(Object.assign({}, this.settings.inputSettings), { meta: Object.assign(Object.assign({}, this.settings.inputSettings.meta), { key // Key used in our input handler
                }) }),
            __value: value
        });
        forceUpdate(this);
    }
    async componentWillLoad() {
        // Parse settings
        if (this.__settings) {
            let settings = this.__settings;
            if (typeof settings === 'string') {
                settings = JSON.parse(settings);
            }
            this.settings = overrideSettings(this.settings, settings);
        }
        // Parse list of input strings and create input elements
        if (this.__inputs) {
            this.inputs = new Map();
            let inputs = this.__inputs;
            if (typeof inputs === 'string') {
                inputs = JSON.parse(inputs);
            }
            inputs.forEach(input => {
                this.addInput(input);
            });
        }
    }
    render() {
        const { containerClass, containerStyle, inputContainerClass, inputContainerStyle, buttonContainerClass, buttonContainerStyle, deleteButtonClass, deleteButtonStyle, addButtonClass, addButtonStyle } = this.settings;
        return (h("div", { key: 'cbcd1a0d3de4be8e99b95d02075d1031451ff249', class: containerClass, style: containerStyle }, [...this.inputs.entries()].map(([key, input]) => {
            return (h("div", { class: inputContainerClass, style: inputContainerStyle }, h("fdi-text-input", Object.assign({ id: key }, input)), h("div", { class: buttonContainerClass, style: buttonContainerStyle }, h("button", { onClick: () => this.removeInput(key), class: deleteButtonClass, style: deleteButtonStyle }, "Delete"))));
        }), h("button", { key: '02105750562a9bfbf311e7b7dd6e17098eca6c43', onClick: () => this.addInput(), class: addButtonClass, style: addButtonStyle }, "Add")));
    }
    static get is() { return "fdi-multi-input"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["fdi-multi-input.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["fdi-multi-input.css"]
        };
    }
    static get properties() {
        return {
            "__inputs": {
                "type": "any",
                "mutable": false,
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__inputs",
                "reflect": false,
                "defaultValue": "'[]'"
            },
            "__settings": {
                "type": "any",
                "mutable": false,
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__settings",
                "reflect": false
            }
        };
    }
    static get states() {
        return {
            "inputs": {},
            "settings": {}
        };
    }
    static get events() {
        return [{
                "method": "multiInputChanged",
                "name": "fdi_wc_multi-input_change",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }];
    }
    static get listeners() {
        return [{
                "name": "fdi_wc_input_change",
                "method": "fdi_wc_input_change__handler",
                "target": undefined,
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=fdi-multi-input.js.map
