import { h } from "@stencil/core";
import { overrideSettings } from "../../utils/utils";
import { v4 as uuidv4 } from "uuid";
export class FDIMultiselect {
    constructor() {
        this.__value = '';
        this.__settings = undefined;
        this.__opts = [];
        this.__selected = undefined;
        this.inputValue = '';
        this.inputFocused = false;
        this.selected = [];
        this.opts = [];
        this.showOptions = false;
        this.settings = {
            disabled: false,
            tagMode: true,
            sortOptions: true,
            selectAllButton: {
                text: 'Select All',
                enabled: true,
                style: {
                    outline: 'solid 1px #ccc',
                    background: 'white',
                    color: '#5f5f5f'
                }
            },
            removeAllButton: {
                text: 'Remove All',
                enabled: true,
                style: {
                    outline: 'solid 1px #ccc',
                    background: 'white',
                    color: '#5f5f5f'
                }
            },
            addCustomButton: {
                text: 'Add Custom Value',
                enabled: true,
                style: {
                    outline: 'solid 1px #ccc',
                    background: 'white',
                    color: '#5f5f5f'
                }
            },
            tooltipEnabled: false,
            singleSelect: false,
            defaults: [],
            style: {
                general: {
                    fontSize: '15px',
                    maxWidth: '100%',
                    width: '100%',
                    margin: 0,
                    outline: '1px solid #ccc',
                    background: 'white',
                },
                optionSelection: {
                    width: '100%',
                    maxHeight: '200px',
                },
            },
            input: {
                placeholder: 'Enter Value',
                fontSize: '15px',
            },
            meta: {},
        };
    }
    fdi_wc_input_change__handler(ev) {
        let data = ev.detail;
        this.inputValue = data.newValue;
    }
    handleKeyDown(ev) {
        if (ev.key === 'Enter' && this.inputFocused) {
            this.addCustomValue();
        }
    }
    emitData() {
        let selected = this.selected.map((selected) => {
            return this.isObj(selected) ? selected.value : selected;
        });
        this.multiselectChanged.emit({
            newValue: selected,
            meta: this.settings.meta,
        });
    }
    onInputFocus() {
        this.showOptions = true;
        this.inputFocused = true;
    }
    onInputBlur() {
        setTimeout(() => {
            this.showOptions = false;
            this.inputFocused = false;
        }, 250);
    }
    inSelected(o) {
        for (const selected of this.selected) {
            if (JSON.stringify(selected) === JSON.stringify(o)) {
                return true;
            }
        }
        return false;
    }
    inOpts(val) {
        for (const [i, opt] of this.opts.entries()) {
            let optVal = this.isObj(opt) ? opt.value : opt;
            if (optVal === val) {
                return i;
            }
        }
        return -1;
    }
    optionsClickHandler(o) {
        if (!this.inSelected(o)) {
            if (this.settings.singleSelect) {
                this.selected = [
                    ...[o]
                ];
            }
            else {
                this.selected = [
                    ...this.selected,
                    ...[o]
                ];
            }
            this.emitData();
        }
    }
    selectAllOpts() {
        this.selected = [
            ...this.opts
        ];
        this.emitData();
    }
    removeAllOpts() {
        this.selected = [
            ...[]
        ];
        this.emitData();
    }
    removeSelectionHandler(i) {
        let selectedArr = this.selected;
        selectedArr.splice(i, 1);
        this.selected = [...selectedArr];
        this.emitData();
    }
    addCustomValue() {
        if (this.inputValue && !this.inSelected(this.inputValue)) {
            if (this.settings.singleSelect) {
                this.selected = [
                    ...[this.inputValue],
                ];
            }
            else {
                this.selected = [
                    ...this.selected,
                    ...[this.inputValue],
                ];
            }
            this.emitData();
        }
    }
    componentWillLoad() {
        let settings = this.__settings;
        let selected = this.__selected;
        let opts = this.__opts;
        if (typeof settings === 'string') {
            settings = JSON.parse(settings);
        }
        if (typeof selected === 'string') {
            selected = JSON.parse(selected);
        }
        if (typeof opts === 'string') {
            opts = JSON.parse(opts);
        }
        this.settings = overrideSettings(this.settings, settings);
        selected = selected || [];
        selected = selected.filter((s) => {
            return s !== null && s !== undefined && s.toString().trim() !== '';
        });
        this.selected = [
            ...(selected || [])
        ];
        if (this.settings.sortOptions) {
            opts.sort((a, b) => {
                a = String(a).toUpperCase();
                b = String(b).toUpperCase();
                if (a > b)
                    return 1;
                if (a < b)
                    return -1;
                return 0;
            });
        }
        this.opts = [
            ...(opts || [])
        ];
        let defaults = Object.values(this.settings.defaults || []) || [];
        let defaultsLength = defaults.length;
        let originalSelectedLength = this.selected.length;
        this.selected.forEach((selected, i) => {
            let selectedIdx = this.inOpts(selected);
            if (selectedIdx > -1) {
                this.selected.splice(i, 1);
                this.selected = [
                    ...this.selected,
                    ...[this.opts[selectedIdx]]
                ];
            }
            else {
                this.opts = [
                    ...this.opts,
                    ...[selected],
                ];
            }
        });
        for (const opt of this.opts) {
            let optVal = this.isObj(opt) ? opt.value : opt;
            if (originalSelectedLength === 0 && defaultsLength > 0 && defaults.includes(optVal)) {
                this.selected = [
                    ...this.selected,
                    ...[opt],
                ];
            }
        }
        this.emitData();
    }
    isObj(variable) {
        return {}.toString.call(variable) === '[object Object]' && variable !== null;
    }
    render() {
        let optionSelectionSettings = Object.assign(Object.assign({}, this.settings.style.optionSelection), {
            display: this.showOptions ? 'block' : 'none',
        });
        /**
         * Set the choices.
         */
        let options = this.opts.map((op) => {
            let isObject = this.isObj(op);
            let opVal = isObject ? op.value : op;
            let opDisplay = isObject ? op.display : op;
            if (this.inputValue) {
                if (!opVal.toLowerCase().includes(this.inputValue.toLowerCase()))
                    return;
            }
            let optionClasses = ['option'];
            if (this.inSelected(op)) {
                optionClasses.push('selected');
            }
            return (h("div", { onClick: () => this.optionsClickHandler(op), class: optionClasses.join(' ') }, h("span", { innerHTML: opDisplay }, " ")));
        });
        /**
         * Set the menu.
         */
        let menu = '';
        if (!this.settings.disabled) {
            menu = (h("div", { class: 'menu' }, !this.settings.singleSelect && this.opts.length > 0 && this.settings.selectAllButton.enabled &&
                h("div", { onClick: () => this.selectAllOpts(), style: this.settings.selectAllButton.style }, this.settings.selectAllButton.text), this.settings.removeAllButton.enabled && this.selected.length > 0 &&
                h("div", { onClick: () => this.removeAllOpts(), style: this.settings.removeAllButton.style }, this.settings.removeAllButton.text), this.settings.addCustomButton.enabled && this.inputValue &&
                h("div", { onClick: () => this.addCustomValue(), style: this.settings.addCustomButton.style }, this.settings.addCustomButton.text)));
        }
        let optionSelection = (h("div", { id: 'optionsContainer', style: optionSelectionSettings }, options));
        let selectedChoices = this.selected.map((selected, i) => {
            let selectedClasses = [
                'selected',
            ];
            let isObj = this.isObj(selected);
            this.settings.tagMode ? selectedClasses.push('tag-mode') : selectedClasses.push('list-mode');
            let tooltipSettings = {
                tooltipText: isObj ? (selected.tooltip || selected.display) : selected,
            };
            return (h("div", { class: selectedClasses.join(' ') }, this.settings.tooltipEnabled &&
                h("fdi-tooltip", { key: uuidv4(), __settings: tooltipSettings }), !this.settings.disabled &&
                h("div", { onClick: () => this.removeSelectionHandler(i), class: 'removeSelected', innerHTML: '&times;' }), h("div", { class: 'selectedContent' }, isObj ? selected.display : selected)));
        });
        let selectedContainerStyle = {
            flexFlow: this.settings.tagMode ? 'wrap' : 'column',
        };
        let selected = (h("div", { id: 'selectedContainer', style: selectedContainerStyle }, selectedChoices));
        return (h("div", { class: 'fdi wc multiselect', style: this.settings.style.general }, !this.settings.disabled &&
            h("div", null, h("fdi-text-input", { onBlur: () => { this.onInputBlur(); }, onFocus: () => { this.onInputFocus(); }, __settings: this.settings.input })), menu, optionSelection, selected));
    }
    static get is() { return "fdi-multiselect"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["fdi-multiselect.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["fdi-multiselect.css"]
        };
    }
    static get properties() {
        return {
            "__value": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__value",
                "reflect": false,
                "defaultValue": "''"
            },
            "__settings": {
                "type": "any",
                "mutable": false,
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__settings",
                "reflect": false
            },
            "__opts": {
                "type": "any",
                "mutable": false,
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__opts",
                "reflect": false,
                "defaultValue": "[]"
            },
            "__selected": {
                "type": "any",
                "mutable": false,
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__selected",
                "reflect": false
            }
        };
    }
    static get states() {
        return {
            "inputValue": {},
            "inputFocused": {},
            "selected": {},
            "opts": {},
            "showOptions": {},
            "settings": {}
        };
    }
    static get events() {
        return [{
                "method": "multiselectChanged",
                "name": "fdi_wc_multiselect_change",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }];
    }
    static get listeners() {
        return [{
                "name": "fdi_wc_input_change",
                "method": "fdi_wc_input_change__handler",
                "target": undefined,
                "capture": false,
                "passive": false
            }, {
                "name": "keydown",
                "method": "handleKeyDown",
                "target": "document",
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=fdi-multiselect.js.map
