import { h, Host } from "@stencil/core";
import { overrideSettings } from "../../utils/utils";
import { v4 as uuidv4 } from "uuid";
export class FDITextInput {
    constructor() {
        this.__value = '';
        this.__settings = undefined;
        this.uuid = undefined;
        this.value = undefined;
        this.oldValue = undefined;
        this.settings = {
            growBasedOnInput: false,
            disabled: false,
            placeholder: '',
            style: false, // Set as object to use those styles
            class: 'fdi wc text',
            fontSize: '20px',
            minWidth: '100px',
            resize: 'vertical',
            flex: 1,
            margin: 0,
            meta: {},
            validation: {
                allowedValues: {
                    value: [],
                    html: ''
                },
                allowEmptyValues: {
                    value: true,
                    html: ''
                },
            },
            useTextArea: false,
        };
    }
    handleChange(e) {
        this.value = e.target.value;
        this.inputChanged.emit({
            newValue: this.value,
            oldValue: this.oldValue,
            hasError: this.getErrorData().error,
            meta: this.settings.meta,
            uuid: this.uuid,
        });
    }
    getErrorData() {
        const value = this.value || this.__value;
        // Checks whether empty values are allowed, and validates value against that condition.
        let invalidValue = !!(!value && !this.settings.validation.allowEmptyValues.value);
        if (invalidValue) {
            return {
                error: true,
                msg: this.settings.validation.allowEmptyValues.html,
            };
        }
        // Checks whether there is a range of valid values, and validates value against valid values. 
        invalidValue = !!(value && this.settings.validation.allowedValues.value.length && !this.settings.validation.allowedValues.value.includes(value));
        if (invalidValue) {
            return {
                error: true,
                msg: this.settings.validation.allowedValues.html,
            };
        }
        return {
            error: false,
            msg: ''
        };
    }
    componentWillLoad() {
        let settings = this.__settings;
        if (typeof settings === 'string') {
            settings = JSON.parse(this.__settings);
        }
        this.settings = overrideSettings(this.settings, settings);
        this.uuid = uuidv4();
    }
    render() {
        let value = this.__value;
        // If style isn't specified, then use the other settings
        let style = this.settings.style;
        if (!style) {
            style = {
                fontSize: this.settings.fontSize,
                minWidth: this.settings.minWidth,
                width: this.settings.width,
                margin: this.settings.margin,
                resize: this.settings.resize,
                flex: this.settings.flex,
            };
            if (this.settings.growBasedOnInput) {
                let val = this.value || value;
                style.width = `${val.length * parseInt(this.settings.fontSize) / 2.10}px`;
            }
        }
        return (h(Host, { key: '8b283a898c8baa22fd3c3ee38ee6a54d6fa38282' }, this.settings.useTextArea ? (h("textarea", { class: this.settings.class, style: this.getErrorData().error ? Object.assign(Object.assign({}, style), { background: 'red', color: 'white' }) : style, placeholder: this.settings.placeholder, disabled: this.settings.disabled, value: value, onInput: (e) => { this.handleChange(e); } })) : (h("input", { class: this.settings.class, style: this.getErrorData().error ? Object.assign(Object.assign({}, style), { background: 'red', color: 'white' }) : style, placeholder: this.settings.placeholder, disabled: this.settings.disabled, value: value, onInput: (e) => { this.handleChange(e); } })), this.getErrorData().error &&
            h("div", { key: 'ca48c106b2c7ae15c94327f95d5f60f49ea1f264', style: { fontSize: '12px', background: 'red', color: 'white', padding: '5px' }, innerHTML: this.getErrorData().msg || 'Invalid value.' })));
    }
    static get is() { return "fdi-text-input"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["fdi-text-input.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["fdi-text-input.css"]
        };
    }
    static get properties() {
        return {
            "__value": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__value",
                "reflect": false,
                "defaultValue": "''"
            },
            "__settings": {
                "type": "any",
                "mutable": false,
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__settings",
                "reflect": false
            }
        };
    }
    static get states() {
        return {
            "uuid": {},
            "value": {},
            "oldValue": {},
            "settings": {}
        };
    }
    static get events() {
        return [{
                "method": "inputChanged",
                "name": "fdi_wc_input_change",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }];
    }
}
//# sourceMappingURL=fdi-text-input.js.map
