import { h } from "@stencil/core";
import { v4 as uuidv4 } from "uuid";
import { overrideSettings } from "../../utils/utils";
import Swal from "sweetalert2";
export class FDITooltip {
    constructor() {
        this.__settings = undefined;
        this.__html = undefined;
        this.id = undefined;
        this.displayTooltip = false;
        this.settings = {
            tooltipText: '',
            openInModal: false,
            showOnClick: false,
            meta: {},
            style: {
                general: {
                    fontSize: '15px',
                    color: 'black',
                    padding: '5px',
                    bottom: 'calc( 100% + 5px )'
                },
            }
        };
    }
    componentWillLoad() {
        let settings = this.__settings;
        if (settings && typeof settings === 'string') {
            settings = JSON.parse(settings);
        }
        if (!settings) {
            settings = this.settings;
        }
        const html = this.__html;
        if (html && html.trim()) {
            settings.tooltipText = html.trim();
        }
        this.settings = overrideSettings(this.settings, settings);
        this.id = uuidv4();
    }
    componentDidLoad() {
        let parent = this.shadowRoot.parentElement;
        if (this.settings.showOnClick) {
            parent.addEventListener('click', (e) => {
                e.preventDefault();
                if (this.displayTooltip) {
                    this.hideTooltip();
                }
                else {
                    this.showTooltip();
                }
            });
        }
        else {
            parent.addEventListener('mouseover', () => {
                this.showTooltip();
            });
            parent.addEventListener('mouseout', () => {
                this.hideTooltip();
            });
        }
    }
    showTooltip() {
        this.displayTooltip = true;
    }
    hideTooltip() {
        this.displayTooltip = false;
    }
    render() {
        if (this.settings.openInModal && this.displayTooltip) {
            Swal.fire({
                html: this.settings.tooltipText,
                showConfirmButton: false,
                showClass: {
                    popup: ''
                },
                padding: '0 5px 5px 5px',
                customClass: {
                    container: 'modal'
                },
                didClose: () => {
                    this.displayTooltip = false;
                },
                willOpen: () => {
                    const styleId = 'fdi-wc-tooltip-swal-css-rules';
                    const existing = document.getElementById(styleId);
                    if (!existing) {
                        const opts = [
                            '.swal2-popup { border-radius: 0 !important; }',
                        ];
                        const style = document.createElement('style');
                        style.innerHTML = opts.join('');
                        style.id = styleId;
                        document.body.appendChild(style);
                    }
                }
            });
            return;
        }
        let containerSettings = {
            display: this.displayTooltip ? 'block' : 'none'
        };
        return (h("div", { id: this.id, class: 'fdi wc tooltip', style: containerSettings }, h("span", { style: this.settings.style.general, innerHTML: this.settings.tooltipText }, " ")));
    }
    static get is() { return "fdi-tooltip"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["fdi-tooltip.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["fdi-tooltip.css"]
        };
    }
    static get properties() {
        return {
            "__settings": {
                "type": "any",
                "mutable": false,
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__settings",
                "reflect": false
            },
            "__html": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__html",
                "reflect": false
            }
        };
    }
    static get states() {
        return {
            "id": {},
            "displayTooltip": {},
            "settings": {}
        };
    }
    static get elementRef() { return "shadowRoot"; }
}
//# sourceMappingURL=fdi-tooltip.js.map
