import { h } from "@stencil/core";
import axios from "axios";
import { v4 as uuidv4 } from "uuid";
export class FDIWizard {
    constructor() {
        this.__targetFI = undefined;
        this.__ajaxUrl = '/wp-admin/admin-ajax.php';
        this.__templateId = undefined;
        this.__existingAnswers = undefined;
        this.__wizardId = undefined;
        this.__master = undefined;
        this.wizardId = undefined;
        this.wizard = null;
        this.templateLoaded = false;
        this.noTemplateText = 'Loading...';
        this.wizardComplete = false;
        this.currentQuestionIndexes = [];
        this.searchQuery = '';
        this.slides = [];
        this.currentSlide = 0;
        this.existingAnswers = null;
        this.renderTrigger = true;
        this.multiSelectOptions = {
            showSelectedText: 'Show Selected',
            hideSelectedText: 'Hide Selected'
        };
        this.textInputValues = {};
    }
    finish() {
        let formatted_answers = {};
        if (this.wizard && this.wizard.hasOwnProperty('questions')) {
            this.wizard.questions.forEach((q) => {
                formatted_answers[q.fi] = q.answer || [];
            });
        }
        this.wizardComplete = true;
        this.wizardCompleted.emit({
            is_complete: this.wizardComplete,
            wizard: this.wizard,
            formatted_answers,
            wizardId: this.wizardId,
            master: this.__master
        });
    }
    fdi_wc_input_change__handler(ev) {
        let customValuePrefix = 'fdi_wizard_custom_value_';
        let searchQueryId = 'fdi_wizard_search_query';
        let id = ev.target.id;
        const { detail } = ev;
        if (detail.meta && detail.meta.wizardId && detail.meta.wizardId === this.wizardId) {
            this.wizard.questions[detail.meta.idx].answer = [detail.newValue];
        }
        if (id) {
            if (id.includes(customValuePrefix)) {
                console.log('if');
                let questionNum = id.replace(customValuePrefix, '');
                this.textInputValues[id] = {
                    newValue: ev.detail.newValue,
                    questionNum,
                };
            }
            else if (id === searchQueryId) {
                console.log('elseif');
                this.searchQuery = ev.detail.newValue;
                this.textInputValues[id] = {
                    newValue: ev.detail.newValue,
                };
            }
        }
    }
    fdi_wc_multiInput_change__handler(ev) {
        const { detail = false } = ev;
        if (detail) {
            const { newValue, meta: { wizardId = false, type = false, idx = false } = {} } = detail;
            if (wizardId === this.wizardId && type === 'multi_input' && idx) {
                this.wizard.questions[idx].answer = [
                    ...newValue,
                ];
            }
        }
    }
    fdi_wc_multiselect_change__handler(ev) {
        let data = ev.detail;
        if (data) {
            let meta = data.meta;
            if (meta.wizardId && meta.wizardId === this.wizardId) {
                if (meta.type && meta.type === 'multiselect' && meta.hasOwnProperty('idx')) {
                    this.wizard.questions[meta.idx].answer = [
                        ...data.newValue,
                    ];
                }
            }
        }
    }
    fdi_wc_datetimepicker_change__handler(ev) {
        let data = ev.detail;
        if (data) {
            let meta = data.meta;
            if (meta.wizardId && meta.wizardId === this.wizardId) {
                if (meta.type && meta.type === 'datetimepicker' && meta.hasOwnProperty('idx')) {
                    this.wizard.questions[meta.idx].answer = [data.newValue];
                }
            }
        }
    }
    fdi_wc_image_uploader_load__handler(ev) {
        let data = ev.detail;
        console.log({ data });
        if (data) {
            let meta = data.meta;
            if (meta.wizardId && meta.wizardId === this.wizardId) {
                if (meta.type && meta.type === 'image' && meta.hasOwnProperty('idx') && meta.wizardId === this.wizardId) {
                    this.wizard.questions[meta.idx].answer = [data.src];
                }
            }
        }
        console.log({ lewizards: this.wizard });
    }
    addCustomAnswer(idx, id) {
        if (this.wizard &&
            this.wizard.hasOwnProperty('questions') &&
            Array.isArray(this.wizard.questions) &&
            this.wizard.questions.hasOwnProperty(idx)) {
            if (this.textInputValues.hasOwnProperty(id)) {
                let value = this.textInputValues[id].newValue;
                let qs = this.wizard.questions;
                if (qs && qs.hasOwnProperty(idx)) {
                    let ops = qs[idx].options.data;
                    if (!ops.includes(value))
                        qs[idx].options.data.push(value);
                    if (!qs[idx].answer.includes(value))
                        qs[idx].answer.push(value);
                    this.renderTrigger = !this.renderTrigger;
                }
            }
        }
    }
    addAllSelections(question) {
        question.answer = JSON.parse(JSON.stringify(question.options.data));
        this.renderTrigger = !this.renderTrigger;
    }
    removeAllSelections(question) {
        question.answer = [];
        this.renderTrigger = !this.renderTrigger;
    }
    removeAnswer(idx, answers) {
        answers.splice(idx, 1);
        this.renderTrigger = !this.renderTrigger;
    }
    showAnswers(question) {
        question.show_answers = true;
        this.renderTrigger = !this.renderTrigger;
    }
    hideAnswers(question) {
        question.show_answers = false;
        this.renderTrigger = !this.renderTrigger;
    }
    processExistingAnswers() {
        if (this.existingAnswers &&
            this.wizard &&
            this.wizard.hasOwnProperty('questions') &&
            Array.isArray(this.wizard.questions)) {
            this.wizard.questions.forEach((q) => {
                if (this.existingAnswers.hasOwnProperty(q.fi))
                    this.existingAnswers[q.fi].forEach((a) => {
                        if (!q.answer.includes(a))
                            q.answer.push(a);
                    });
            });
        }
    }
    loadSlides() {
        let questionsPerSlide = this.wizard.questions_per_slide;
        let questionCount = this.wizard.questions.length;
        let indexArr = [];
        for (let j = 0; j < questionCount; j++) {
            indexArr.push(j);
        }
        for (let i = 0; i < questionCount; i += questionsPerSlide) {
            this.slides.push(indexArr.slice(i, i + questionsPerSlide));
        }
    }
    goToInitialSlide() {
        if (this.__targetFI) {
            this.skipToFriendlyIndex(this.__targetFI);
        }
        else {
            this.moveToSlideContainingIndex(0);
        }
    }
    async loadTemplate() {
        let data = new URLSearchParams();
        data.append('action', 'fdi_fetch_wizard');
        data.append('templateId', this.__templateId);
        console.log({
            ajaxurl: this.__ajaxUrl,
            data
        });
        axios.post(this.__ajaxUrl, data).then((r) => {
            console.log({
                wizardResponse: r,
            });
            if (r.hasOwnProperty('data') && r.data.hasOwnProperty('wizard')) {
                this.wizard = r.data.wizard;
                this.templateLoaded = true;
                this.processExistingAnswers();
                this.loadSlides();
                this.goToInitialSlide();
            }
        }).catch((err) => {
            console.log({
                wizardError: err,
            });
        });
        //   let test = {
        //     "title": "Comparison Page Data",
        //     "finish_wizard_text": "Submit",
        //     "questions_per_slide": 5,
        //     "questions": [
        //         {
        //             "text1": "Header Title",
        //             "text2": "",
        //             "fi": "header_title",
        //             "is_required": false,
        //             "allow_custom_answer": true,
        //             "answer": [],
        //             "default_answer": [],
        //             "show_answers": false,
        //             "type": "text",
        //             "options": {
        //                 "source_type": null,
        //                 "source": null,
        //                 "data": null,
        //                 "data_type": null
        //             }
        //         },
        //         {
        //             "text1": "Header Paragraph",
        //             "text2": "",
        //             "fi": "header_paragraph",
        //             "is_required": false,
        //             "allow_custom_answer": true,
        //             "answer": [],
        //             "default_answer": [],
        //             "show_answers": false,
        //             "type": "text",
        //             "settings": {
        //               "useTextArea": true
        //             },
        //             "options": {
        //                 "source_type": null,
        //                 "source": null,
        //                 "data": null,
        //                 "data_type": null
        //             }
        //         },
        //         {
        //             "text1": "Left Vehicle Image",
        //             "text2": "",
        //             "fi": "left_vehicle_image",
        //             "is_required": false,
        //             "allow_custom_answer": true,
        //             "answer": [],
        //             "default_answer": [],
        //             "show_answers": false,
        //             "type": "image",
        //             "options": {
        //                 "source_type": null,
        //                 "source": null,
        //                 "data": null,
        //                 "data_type": null
        //             }
        //         },
        //         {
        //             "text1": "Right Vehicle Image",
        //             "text2": "",
        //             "fi": "right_vehicle_image",
        //             "is_required": false,
        //             "allow_custom_answer": true,
        //             "answer": [],
        //             "default_answer": [],
        //             "show_answers": false,
        //             "type": "image",
        //             "options": {
        //                 "source_type": null,
        //                 "source": null,
        //                 "data": null,
        //                 "data_type": null
        //             }
        //         },
        //         {
        //             "text1": "H2 Paragraphs",
        //             "text2": "",
        //             "fi": "paragraphs",
        //             "is_required": false,
        //             "answer": [],
        //             "default_answer": [],
        //             "show_answers": false,
        //             "type": "multi_input",
        //             "settings": {
        //                 "inputSettings": {
        //                     "useTextArea": true
        //                 }
        //             },
        //             "options": {
        //                 "source_type": null,
        //                 "source": null,
        //                 "data": [],
        //                 "data_type": "array"
        //             }
        //         }
        //     ]
        // };
        //   this.wizard = test;
        //   this.templateLoaded = true;
        //   this.processExistingAnswers();
        //   this.loadSlides();
        //   this.goToInitialSlide();
    }
    moveToSlideContainingIndex(idx) {
        let slidesAmt = this.slides.length;
        for (let i = 0; i < slidesAmt; i++) {
            if (this.slides[i].includes(idx)) {
                this.currentQuestionIndexes = this.slides[i];
                this.currentSlide = i;
            }
        }
    }
    skipToFriendlyIndex(fi) {
        if (this.wizard && this.wizard.hasOwnProperty('questions') && Array.isArray(this.wizard.questions)) {
            let foundIndex = false;
            let questionsLength = this.wizard.questions.length;
            for (let i = 0; i < questionsLength; i++) {
                let q = this.wizard.questions[i];
                if ((q.is_required && q.answer.length === 0) || q.fi === fi) {
                    this.moveToSlideContainingIndex(i);
                    foundIndex = true;
                    break;
                }
            }
            if (!foundIndex) {
                this.moveToSlide(0);
            }
        }
    }
    matchesSearchQuery(string) {
        if (this.searchQuery === '')
            return true;
        let rgx = new RegExp(this.searchQuery.toLowerCase(), 'g');
        return String(string).toLowerCase().match(rgx);
    }
    moveToSlide(idx) {
        this.currentQuestionIndexes = this.slides[idx];
        this.currentSlide = idx;
    }
    nextSlide() {
        this.moveToSlide(this.currentSlide + 1);
    }
    prevSlide() {
        this.moveToSlide(this.currentSlide - 1);
    }
    async componentWillLoad() {
        this.wizardId = this.__wizardId ? this.__wizardId : uuidv4();
        if (this.__existingAnswers) {
            this.existingAnswers = JSON.parse(this.__existingAnswers);
        }
        await this.loadTemplate();
    }
    handleAnswer(e, i) {
        if (this.wizard &&
            this.wizard.hasOwnProperty('questions') &&
            this.wizard.questions.hasOwnProperty(i) &&
            this.wizard.questions[i].hasOwnProperty('type')) {
            let q = this.wizard.questions[i];
            if (q.type === 'radio') {
                let value = e.target.value;
                value = value === 'true';
                q.answer = [value];
            }
            this.renderTrigger = !this.renderTrigger;
        }
    }
    render() {
        let _currentQuestionsObj = {};
        let _currentQuestions = [];
        let _currentQuestionsKeys = [];
        let _canGoToNextSlide = false;
        let _canGoToPrevSlide = false;
        let _canFinishWizard = false;
        let question = (h("div", { class: 'question' }));
        let templateLoaded = '';
        let templateNotLoaded = (h("div", null, this.noTemplateText));
        let wizardTitle = '';
        let searchInput = '';
        if (this.templateLoaded && this.wizard && this.wizard.hasOwnProperty('questions') && Array.isArray(this.wizard.questions)) {
            let length = this.wizard.questions.length;
            let added = 0;
            let questionsPerSlide = this.wizard.questions_per_slide;
            for (let i = 0; i < length; i++) {
                if (added === questionsPerSlide)
                    break;
                if (this.slides[this.currentSlide].includes(i)) {
                    _currentQuestionsObj[i] = this.wizard.questions[i];
                    added++;
                }
            }
            _currentQuestionsKeys = Object.keys(_currentQuestionsObj);
            if (_currentQuestionsKeys.length > 0) {
                _currentQuestions = Object.values(_currentQuestionsObj);
            }
            /**
             * Next slide button
             */
            _canGoToNextSlide = true;
            _currentQuestions.forEach((q) => {
                if (q.answer.length === 0 && q.is_required === true) {
                    _canGoToNextSlide = false;
                    return false;
                }
            });
            if (this.currentQuestionIndexes.includes(this.wizard.questions.length - 1) && !this.wizard.show_review_page) {
                _canGoToNextSlide = false;
            }
            /**
             * Prev slide button
             */
            _canGoToPrevSlide = true;
            _currentQuestions.forEach((q) => {
                if (q.answer.length === 0 && q.is_required === true || this.currentQuestionIndexes.includes(0)) {
                    _canGoToPrevSlide = false;
                    return false;
                }
            });
            /**
             * Finish / submit wizard button
             */
            _canFinishWizard = true;
            this.wizard.questions.forEach((q) => {
                if (q.answer.length === 0 && q.is_required === true) {
                    _canFinishWizard = false;
                    return false;
                }
            });
            question = _currentQuestionsKeys.map((idx) => {
                let i = parseInt(idx);
                let question = this.wizard.questions[i];
                let text1 = '';
                if (question.text1) {
                    text1 = (h("div", { class: 'text1' }, question.text1));
                }
                let text2 = '';
                if (question.text2) {
                    text2 = (h("div", { class: 'text2' }, question.text2));
                }
                let questionContent = '';
                if (question.type === 'text') {
                    const inputSettings = Object.assign(Object.assign({}, question === null || question === void 0 ? void 0 : question.settings), { placeholder: "Enter text", meta: {
                            wizardId: this.wizardId,
                            idx,
                            type: 'text'
                        } });
                    let input = (h("fdi-text-input", { key: idx, __settings: inputSettings, __value: question.answer }));
                    questionContent = (h("div", { class: 'content' }, input));
                }
                else if (question.type === 'multi_input') {
                    const inputSettings = Object.assign(Object.assign({}, question === null || question === void 0 ? void 0 : question.settings), { meta: {
                            wizardId: this.wizardId,
                            idx,
                            type: 'multi_input'
                        } });
                    let input = (h("fdi-multi-input", { key: idx, __inputs: question.answer, __settings: inputSettings }));
                    questionContent = (h("div", { class: 'content' }, input));
                }
                else if (question.type === 'image') {
                    const imageSettings = {
                        meta: {
                            type: 'image',
                            wizardId: this.wizardId,
                            idx,
                        }
                    };
                    let imageUpload = (h("fdi-image-uploader", { __settings: imageSettings, __value: question.answer[0] }));
                    questionContent = (h("div", { class: 'content' }, imageUpload));
                }
                else if (question.type === 'multi_select' && Array.isArray(question.options.data)) {
                    let multiselectSettings = {
                        tagMode: false,
                        input: {
                            placeholder: 'Enter Value or Search'
                        },
                        meta: {
                            type: 'multiselect',
                            wizardId: this.wizardId,
                            idx,
                        }
                    };
                    let multiselect = (h("fdi-multiselect", { key: idx, __settings: multiselectSettings, __opts: question.options.data, __selected: question.answer }));
                    questionContent = (h("div", { class: 'content' }, multiselect));
                }
                else if (question.type === 'radio' && typeof question.options.data === 'object') {
                    let radioOptions = Object.entries(question.options.data).map((radio) => {
                        let display = radio[0];
                        let value = radio[1];
                        if (question.answer.length === 0 &&
                            question.hasOwnProperty('default_answer') &&
                            Array.isArray(question.default_answer) &&
                            question.default_answer.length > 0) {
                            question.answer = JSON.parse(JSON.stringify(question.default_answer));
                        }
                        let selected = question.answer.includes(value);
                        return (h("div", null, h("label", null, display), h("input", { checked: selected, name: question.fi, type: 'radio', value: value, onChange: (e) => this.handleAnswer(e, i) })));
                    });
                    questionContent = (h("div", { class: 'content' }, h("div", { class: 'radioOptions' }, radioOptions)));
                }
                else if (question.type === 'datetimepicker') {
                    let datetimepicker = (h("fdi-date-time-picker", { key: idx, __datetime: question.answer[0], __dateonly: question.options.date_only, __settings: {
                            meta: {
                                type: 'datetimepicker',
                                wizardId: this.wizardId,
                                idx,
                            }
                        } }));
                    questionContent = (h("div", null, datetimepicker));
                }
                let prevButtonClass = 'navButton';
                if (!_canGoToPrevSlide)
                    prevButtonClass += ' grayColor noPointerEvents';
                let prevButton = (h("div", { class: prevButtonClass, onClick: () => this.prevSlide() }, h("span", null, "Previous")));
                let finishWizardText = this.wizard.finish_wizard_text || 'Finish';
                let submitButtonClass = 'navButton';
                if (!_canFinishWizard)
                    submitButtonClass += ' grayColor noPointerEvents';
                let submitButton = (h("div", { class: submitButtonClass, onClick: () => this.finish() }, h("span", null, finishWizardText)));
                let nextButtonClass = 'navButton';
                if (!_canGoToNextSlide)
                    nextButtonClass += ' grayColor noPointerEvents';
                let nextButton = (h("div", { class: nextButtonClass, onClick: () => this.nextSlide() }, h("span", null, "Next")));
                let navigationContainer = (h("div", { class: 'navigationContainer' }, prevButton, submitButton, nextButton));
                return (h("div", { class: 'question' }, h("div", { class: 'titleContainer' }, text1, text2), questionContent, navigationContainer));
            });
            if (this.wizard.title) {
                wizardTitle = (h("div", { class: 'titleContainer' }, h("h2", { class: 'title' }, this.wizard.title)));
            }
            templateLoaded = (h("div", { class: 'fdi', id: 'wizard' }, wizardTitle, searchInput, question));
            templateNotLoaded = '';
        }
        return (h("div", null, templateLoaded, templateNotLoaded));
    }
    static get is() { return "fdi-wizard"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["fdi-wizard.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["fdi-wizard.css"]
        };
    }
    static get properties() {
        return {
            "__targetFI": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__target-f-i",
                "reflect": false
            },
            "__ajaxUrl": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__ajax-url",
                "reflect": false,
                "defaultValue": "'/wp-admin/admin-ajax.php'"
            },
            "__templateId": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__template-id",
                "reflect": false
            },
            "__existingAnswers": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__existing-answers",
                "reflect": false
            },
            "__wizardId": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__wizard-id",
                "reflect": false
            },
            "__master": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__master",
                "reflect": false
            }
        };
    }
    static get states() {
        return {
            "wizardId": {},
            "wizard": {},
            "templateLoaded": {},
            "noTemplateText": {},
            "wizardComplete": {},
            "currentQuestionIndexes": {},
            "searchQuery": {},
            "slides": {},
            "currentSlide": {},
            "existingAnswers": {},
            "renderTrigger": {},
            "multiSelectOptions": {},
            "textInputValues": {}
        };
    }
    static get events() {
        return [{
                "method": "wizardCompleted",
                "name": "fdi_wc_wizardCompleted",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }];
    }
    static get listeners() {
        return [{
                "name": "fdi_wc_input_change",
                "method": "fdi_wc_input_change__handler",
                "target": undefined,
                "capture": false,
                "passive": false
            }, {
                "name": "fdi_wc_multi-input_change",
                "method": "fdi_wc_multiInput_change__handler",
                "target": undefined,
                "capture": false,
                "passive": false
            }, {
                "name": "fdi_wc_multiselect_change",
                "method": "fdi_wc_multiselect_change__handler",
                "target": undefined,
                "capture": false,
                "passive": false
            }, {
                "name": "fdi_wc_datetimepicker_change",
                "method": "fdi_wc_datetimepicker_change__handler",
                "target": undefined,
                "capture": false,
                "passive": false
            }, {
                "name": "fdi_wc_image_uploader_load",
                "method": "fdi_wc_image_uploader_load__handler",
                "target": undefined,
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=fdi-wizard.js.map
