import { h, forceUpdate } from "@stencil/core";
// import axios from 'axios';
import { v4 as uuidv4 } from "uuid";
export class FdiWizards {
    constructor() {
        this.__wizards = undefined;
        this.__displayFields = undefined;
        this.__title = undefined;
        this.__templateId = undefined;
        this.id = undefined;
        this.wizards = undefined;
        this.displayFields = undefined;
        this.activeWizard = undefined;
    }
    finish() {
        this.wizardsComplete.emit({
            wizards: this.wizards,
        });
    }
    fdi_wc_input_change__handler(ev) {
        const { detail } = ev;
        console.log({ detail });
        if (detail.master === this.id) {
            const currentWizard = this.wizards.get(detail.wizardId);
            if (currentWizard) {
                currentWizard.existingAnswers = detail.formatted_answers;
                this.cancelWizard();
            }
        }
    }
    async componentWillLoad() {
        this.id = uuidv4();
        if (this.__wizards) {
            this.wizards = new Map();
            const temp = JSON.parse(this.__wizards);
            temp.forEach(wizard => {
                this.wizards.set(uuidv4(), wizard);
            });
        }
        if (this.__displayFields) {
            this.displayFields = JSON.parse(this.__displayFields);
        }
    }
    openWizard(key) {
        if (!this.activeWizard) {
            this.activeWizard = key;
        }
    }
    cancelWizard() {
        if (this.activeWizard) {
            this.activeWizard = null;
        }
    }
    removeWizard(key) {
        this.wizards.delete(key);
        forceUpdate(this);
    }
    addWizard() {
        this.wizards.set(uuidv4(), {
            existingAnswers: {},
        });
        forceUpdate(this);
    }
    render() {
        const displays = [...this.wizards.entries()].map(([key, wizard]) => {
            const values = this.displayFields.map(field => {
                let value = wizard.existingAnswers[field] || null;
                if (Array.isArray(value)) {
                    value = value.join(',');
                }
                return (h("div", null, field, ": ", value));
            });
            return (h("div", { class: "wizards-wizard" }, values, h("button", { onClick: () => this.openWizard(key) }, "Edit"), h("button", { onClick: () => this.removeWizard(key) }, "Delete")));
        });
        return (h("div", null, h("h2", null, this.__title), this.activeWizard &&
            h("div", null, h("fdi-wizard", { "__existing-answers": JSON.stringify(this.wizards.get(this.activeWizard).existingAnswers), __master: this.id, "__wizard-id": this.activeWizard, "__template-id": this.__templateId }), h("div", null, h("button", { onClick: () => this.cancelWizard() }, "Cancel"))), !this.activeWizard && displays, !this.activeWizard && h("button", { onClick: () => this.addWizard() }, "Add New"), !this.activeWizard && h("button", { onClick: () => this.finish() }, "Save Changes")));
    }
    static get is() { return "fdi-wizards"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["fdi-wizards.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["fdi-wizards.css"]
        };
    }
    static get properties() {
        return {
            "__wizards": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__wizards",
                "reflect": false
            },
            "__displayFields": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__display-fields",
                "reflect": false
            },
            "__title": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__title",
                "reflect": false
            },
            "__templateId": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "attribute": "__template-id",
                "reflect": false
            }
        };
    }
    static get states() {
        return {
            "id": {},
            "wizards": {},
            "displayFields": {},
            "activeWizard": {}
        };
    }
    static get events() {
        return [{
                "method": "wizardsComplete",
                "name": "fdi_wc_wizardsComplete",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }];
    }
    static get listeners() {
        return [{
                "name": "fdi_wc_wizardCompleted",
                "method": "fdi_wc_input_change__handler",
                "target": undefined,
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=fdi-wizards.js.map
