import { r as registerInstance, c as createEvent, f as forceUpdate, h } from './index-62acac3e.js';
import { v as v4 } from './v4-b0c3c298.js';

const fdiWizardsCss = ".wizards-wizard{display:inline-block;margin:10px;padding:10px;border:solid 1px #ccc}button{cursor:pointer;margin:5px}h2{text-align:center}";
const FdiWizardsStyle0 = fdiWizardsCss;

const FdiWizards = class {
    constructor(hostRef) {
        registerInstance(this, hostRef);
        this.wizardsComplete = createEvent(this, "fdi_wc_wizardsComplete", 7);
        this.__wizards = undefined;
        this.__displayFields = undefined;
        this.__title = undefined;
        this.__templateId = undefined;
        this.id = undefined;
        this.wizards = undefined;
        this.displayFields = undefined;
        this.activeWizard = undefined;
    }
    finish() {
        this.wizardsComplete.emit({
            wizards: this.wizards,
        });
    }
    fdi_wc_input_change__handler(ev) {
        const { detail } = ev;
        console.log({ detail });
        if (detail.master === this.id) {
            const currentWizard = this.wizards.get(detail.wizardId);
            if (currentWizard) {
                currentWizard.existingAnswers = detail.formatted_answers;
                this.cancelWizard();
            }
        }
    }
    async componentWillLoad() {
        this.id = v4();
        if (this.__wizards) {
            this.wizards = new Map();
            const temp = JSON.parse(this.__wizards);
            temp.forEach(wizard => {
                this.wizards.set(v4(), wizard);
            });
        }
        if (this.__displayFields) {
            this.displayFields = JSON.parse(this.__displayFields);
        }
    }
    openWizard(key) {
        if (!this.activeWizard) {
            this.activeWizard = key;
        }
    }
    cancelWizard() {
        if (this.activeWizard) {
            this.activeWizard = null;
        }
    }
    removeWizard(key) {
        this.wizards.delete(key);
        forceUpdate(this);
    }
    addWizard() {
        this.wizards.set(v4(), {
            existingAnswers: {},
        });
        forceUpdate(this);
    }
    render() {
        const displays = [...this.wizards.entries()].map(([key, wizard]) => {
            const values = this.displayFields.map(field => {
                let value = wizard.existingAnswers[field] || null;
                if (Array.isArray(value)) {
                    value = value.join(',');
                }
                return (h("div", null, field, ": ", value));
            });
            return (h("div", { class: "wizards-wizard" }, values, h("button", { onClick: () => this.openWizard(key) }, "Edit"), h("button", { onClick: () => this.removeWizard(key) }, "Delete")));
        });
        return (h("div", null, h("h2", null, this.__title), this.activeWizard &&
            h("div", null, h("fdi-wizard", { "__existing-answers": JSON.stringify(this.wizards.get(this.activeWizard).existingAnswers), __master: this.id, "__wizard-id": this.activeWizard, "__template-id": this.__templateId }), h("div", null, h("button", { onClick: () => this.cancelWizard() }, "Cancel"))), !this.activeWizard && displays, !this.activeWizard && h("button", { onClick: () => this.addWizard() }, "Add New"), !this.activeWizard && h("button", { onClick: () => this.finish() }, "Save Changes")));
    }
};
FdiWizards.style = FdiWizardsStyle0;

export { FdiWizards as fdi_wizards };

//# sourceMappingURL=fdi-wizards.entry.js.map